//+------------------------------------------------------------------+
//|                                                 s-StochTrend.mq4 |
//|                      Copyright  2011, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"

#property indicator_separate_window
#property indicator_minimum    0
#property indicator_maximum    100
#property indicator_buffers    2
#property indicator_color1     MediumBlue
#property indicator_color2     Green
#property indicator_width1     2
#property indicator_width2     2
#property indicator_level1     80
#property indicator_level2     60
#property indicator_level3     40
#property indicator_level4     20
#property indicator_levelcolor Maroon
#property indicator_levelwidth 1
#property indicator_levelstyle STYLE_DOT



extern ENUM_TIMEFRAMES TimeFrame       = PERIOD_CURRENT;
extern int             KPeriod         = 21;
extern int             DPeriod         = 34;
extern int             Slowing         = 3;
extern ENUM_STO_PRICE  PriceField      = 0;
extern ENUM_MA_METHOD  MaMode          = MODE_SMA;
extern double          ZoneHighPer     = 65;
extern double          ZoneLowPer      = 35;
extern bool            alertsOn        = true;
extern bool            alertsOnCurrent = true;
extern bool            alertsMessage   = true;
extern bool            alertsSound     = false;
extern bool            alertsEmail     = false;
extern bool            alertsNotify    = false;
extern string          soundFile       = "alert2.wav";
extern bool            ShowLines       = false;
extern string          linesID         = "s-St lines1 ";
extern color           linesUpColor    = Green;
extern color           linesDnColor    = Crimson;
extern int             linesStyle      = STYLE_SOLID;
extern int             linesWidth      = 2;
extern bool            linesOnFirst    = false;
extern bool            Interpolate     = true;


double sto[];
double sig[];
double trend[];
string indicatorFileName;
bool   returnBars;
string indicatorName;
string shortName;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+

int init()
{
   IndicatorBuffers(3);
   SetIndexBuffer(0,sto);  SetIndexLabel(0,"Stoch");
   SetIndexBuffer(1,sig);  SetIndexLabel(1,"Signal");
   SetIndexBuffer(2,trend);
   
   TimeFrame         = MathMax(TimeFrame,_Period);
   shortName         = linesID+  timeFrameToString(TimeFrame)+" s-StochTrend("+KPeriod+","+DPeriod+","+Slowing+")";
   indicatorFileName = WindowExpertName();
   returnBars        = TimeFrame==-99; 

   IndicatorShortName(shortName);
return(0);
}

int deinit()
{
   string find = linesID+":";
   for (int i=ObjectsTotal()-1; i>= 0; i--)
   {
      string name = ObjectName(i); if (StringFind(name,find)==0) ObjectDelete(name);
   }
   return(0); 
}

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit  = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { sto[0] = limit+1; return(0); }
         if (TimeFrame != _Period)
         {
           limit = (int)MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/_Period));
           for (int i=limit; i>=0; i--)
           {
              int y = iBarShift(NULL,TimeFrame,Time[i]);
                 sto[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,KPeriod,DPeriod,Slowing,MaMode,PriceField,ZoneHighPer,ZoneLowPer,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,soundFile,ShowLines,linesID,linesUpColor,linesDnColor,linesStyle,linesWidth,linesOnFirst,0,y);
                 sig[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,KPeriod,DPeriod,Slowing,MaMode,PriceField,ZoneHighPer,ZoneLowPer,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,soundFile,ShowLines,linesID,linesUpColor,linesDnColor,linesStyle,linesWidth,linesOnFirst,1,y);
                 
                 //
                 //
                 //
                 //
                 //
      
                 if (!Interpolate || y==iBarShift(NULL,TimeFrame,Time[i-1])) continue;

                 //
                 //
                 //
                 //
                 //

                 datetime time = iTime(NULL,TimeFrame,y);
                    for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;
                    for(int x = 1; x < n; x++) 
                    {
                       sto[i+x] = sto[i] + (sto[i+n] - sto[i]) * x/n;
                       sig[i+x] = sig[i] + (sig[i+n] - sig[i]) * x/n;
                    }                                       
      }
   return(0);
   } 
     
   //
   //
   //
   //
   //
   
   int window = WindowFind(shortName);
   for (i=limit; i>=0; i--)
   {
      sto[i] = iStochastic(NULL,0,KPeriod,DPeriod,Slowing,MaMode,PriceField,MODE_MAIN  ,i);
      sig[i] = iStochastic(NULL,0,KPeriod,DPeriod,Slowing,MaMode,PriceField,MODE_SIGNAL,i); 
      
         trend[i] = trend[i+1];
         if ((sto[i]>sig[i]) && (sto[i+1]<sig[i+1]) && (sto[i+1]<ZoneLowPer)  && (sig[i+1]<ZoneLowPer))  trend[i] = 1;
         if ((sto[i]<sig[i]) && (sto[i+1]>sig[i+1]) && (sto[i+1]>ZoneHighPer) && (sig[i+1]>ZoneHighPer)) trend[i] =-1;
      
         //
         //
         //
         //
         //
     
         if (ShowLines && window > -1)
         {
             string name = linesID+":"+Time[i];
             int    add  = 0; if (!linesOnFirst) add = _Period*60-1;
             ObjectDelete(name);
             if (trend[i]!= trend[i+1])
             {
                color theColor  = linesUpColor; if (trend[i]==-1) theColor = linesDnColor;
                   ObjectCreate(name,OBJ_VLINE,window,Time[i]+add,0);
                      ObjectSet(name,OBJPROP_WIDTH,linesWidth);
                      ObjectSet(name,OBJPROP_STYLE,linesStyle);
                      ObjectSet(name,OBJPROP_COLOR,theColor);
              }
            }
      }
      
      //
      //
      //
      //
      //
      
      if (alertsOn)
      {
        if (alertsOnCurrent)
             int whichBar = 0;
        else     whichBar = 1; 
        if (trend[whichBar] != trend[whichBar+1])
        if (trend[whichBar] == 1)
              doAlert("up");
        else  doAlert("down");       
      }              
return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," ",timeFrameToString(_Period)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," s-StochTrend ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," s-StochTrend "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}

